% SOLMAT.m
%
% Computes the RE solution of the model written in Dynare in CLOSED.MOD and
% checks that estimated shocks and parameters recover the data. This is a
% handy diagnostic.

%% Load Results from estimation
load closed_results_01022011
%load closed_results_03022011HighPsi
usdata

%% Options
checkplots = 1; % Check solution through Smats.m
dodecomposition = 1; % Decompose movement in observed series due to shocks
checksum = 1; % Check that the decomposed movements add up to the observed series
calibrated = 0; % If the model was calibrated, rather than estimated results loaded
%% Load parameters at posterior mean and calibrated parameters
beta = 0.99;
psi = 0.1; % Change this for high psi
omega = 0.0617;
if calibrated % Ireland's calibration
rho_a = 0.9470;
rho_e = 0.9625;
omega = 0.0617;
rho_pi = 0.3597;
rho_g = 0.2536;
rho_x = 0.0347;
ppi_ss=0.0086;
g_ss=0.0048;
else   
rho_a = oo_.mle_mode.parameters.rho_a;%posterior_mode.parameters.rho_a;
rho_e = oo_.mle_mode.parameters.rho_e;%oo_.posterior_mode.parameters.rho_e;
%omega = oo_.posterior_mode.parameters.omega;
rho_pi = oo_.mle_mode.parameters.rho_pi;%oo_.posterior_mode.parameters.rho_pi;
rho_g = oo_.mle_mode.parameters.rho_g;%oo_.posterior_mode.parameters.rho_g;
rho_x = oo_.mle_mode.parameters.rho_x;%oo_.posterior_mode.parameters.rho_x;
ppi_ss=ppiss;
g_ss=gss;
end

% Compute slope parameters
ad_slope = -(1+rho_g+rho_x)/rho_pi;
as_slope = psi;
% Compute constants
ad_constant = ppi_ss - ad_slope*g_ss;
as_constant = ppi_ss - as_slope*g_ss;

% Load Matrices in SMATS.m form
% System given as
%          GAM0*y(t)=GAM1*y(t-1)+C+PSI*e(t)+PPI*eta(t),
%   with e(t) an exogenous variable process and eta(t) being endogenously determined
%   one-step-ahead expectational errors.  Returned system is
%         y(t)=S0 + S1*y(t-1)+S2*e(t)+S3*sunspot(t)  .
%
% variables are ordered as follows:
%         1   2   3   4   5      6    7     8    9    10
% y(t) = [a   e   z   x ppihat ghat  yhat  rhat  Etx  Etppi]'
%
%          1     2     3     4
% e(t) = [eps_a eps_e eps_z eps_r]
%
% eta(t) [etax etappi]

if checkplots
nvars = 10;
nshocks = 4;

g0 = zeros(10);
g0(1,1) = 1;
g0(2,2) = 1;
g0(3,3) = 1;
g0(4,1) = -(1-omega)*(1-rho_a);
g0(4,4) = 1;
g0(4,8) = 1;
g0(4,9) = -1;
g0(4,10) = -1;
g0(5,2) = 1;
g0(5,4) = -psi;
g0(5,5) = 1;
g0(5,10) = -beta;
g0(6,1) = omega;
g0(6,4) = 1;
g0(6,7) = -1;
g0(7,3) = -1;
g0(7,6) = 1;
g0(7,7) = -1;
g0(8,4) = -rho_x;
g0(8,5) = -rho_pi;
g0(8,6) = -rho_g;
g0(8,8) = 1;
g0(9,4) = 1;
g0(10,5) = 1;

g1 = zeros(nvars);
g1(1,1) = rho_a;
g1(2,2) = rho_e;
g1(7,7) = -1;
g1(8,8) = 1;
g1(9,9) = 1;
g1(10,10) = 1;

PSI = zeros(nvars,nshocks);
PSI(1,1) = 1;
PSI(2,2) = 1;
PSI(3,3) = 1;
PSI(8,4) = 1;

PPI = zeros(nvars,2);
PPI(9,1) = 1;
PPI(10,2) = 1;

C0 = zeros(nvars,1);

% Compute solution

[S0, S1, S2, S3, unique] = Smats(C0, g0, g1, PSI, PPI);
end
%% Put estimated shocks trough solution
% See Larry Christiano's comment on Smoothed Shocks
% http://www.cepremap.cnrs.fr/juillard/mambo/index.php?option=com_forum&Itemid=95&page=viewtopic&t=1381
T = max(size(ghat));

if calibrated == 0
eps_a = oo_.SmoothedShocks.eps_a;
eps_e = oo_.SmoothedShocks.eps_e;
eps_z = oo_.SmoothedShocks.eps_z;
eps_r = oo_.SmoothedShocks.eps_r;

eps_vec = [eps_a(1:end) eps_e(1:end) eps_z(1:end) eps_r(1:end)]';  
CC=corrcoef(eps_vec');

y_smats = zeros(nvars,T);
y_smats(:,1) = [oo_.SmoothedVariables.a(1);
                oo_.SmoothedVariables.e(1);
                oo_.SmoothedVariables.z(1);
                oo_.SmoothedVariables.x(1);
                oo_.SmoothedVariables.ppihat(1);
                oo_.SmoothedVariables.ghat(1);
                oo_.SmoothedVariables.yhat(1);
                oo_.SmoothedVariables.rhat(1);
                oo_.SmoothedVariables.Ex(1);
                oo_.SmoothedVariables.Eppihat(1)];
            
% Extract demand and supply shifters           
ast = oo_.SmoothedVariables.ast + as_constant;
adt = oo_.SmoothedVariables.adt + ad_constant;

for t = 2:T;
    y_smats(:,t) = S1*y_smats(:,t-1)+S2*eps_vec(:,t);
end
end
%% Compute AS and AD curves
samplesize = max(size(ghat));
gmin = -.02;
gmax = .02;
step = .0005;
gvec = gmin:step:gmax;
[rows cols] = size(gvec); 
% Create tensors to store demand and supply curves
ppi_ad = zeros(rows,cols,samplesize);
ppi_as = zeros(rows,cols,samplesize);

% for t = 1:samplesize
%     ppi_ad(:,:,t) = ad(gvec,ad_slope,adt(t));
%     ppi_as(:,:,t) = as(gvec,as_slope,ast(t));
% end

%% Compute steady state vectors (for ASADPLOT.M)
g_ssvec = g_ss*ones(rows,cols);
ppi_ssvec = ppi_ss*ones(rows,cols);

%% The following plots check that the estimated shocks and parameters
%  recover the data. Set checkplots = 0 to avoid them.
time = 1:T;

if checkplots
figure(1)
plot(time,y_smats(5,:),'b',time,ppihat,'r'), title('Inflation')
figure(2)
plot(time,y_smats(6,:),'b',time,ghat,'r'), title('Growth')
figure(3)
plot(time,y_smats(8,:),'b',time,rhat,'r'), title('Interest Rate')
end

if dodecomposition
% Only demand shocks
y_eps_a = zeros(nvars,T);
y_eps_a(:,1) = y_smats(:,1)/4;
eps_a_vec = [eps_a(1:end) 0*eps_e(1:end) 0*eps_z(1:end) 0*eps_r(1:end)]';  
for t = 2:T;
    y_eps_a(:,t) = S1*y_eps_a(:,t-1)+S2*eps_a_vec(:,t);
end
% Only cost-push shocks
y_eps_e = zeros(nvars,T);
y_eps_e(:,1) = y_smats(:,1)/4;
eps_e_vec = [0*eps_a(1:end) eps_e(1:end) 0*eps_z(1:end) 0*eps_r(1:end)]';  
for t = 2:T;
    y_eps_e(:,t) = S1*y_eps_e(:,t-1)+S2*eps_e_vec(:,t);
end
% Only tech shocks
y_eps_z = zeros(nvars,T);
y_eps_z(:,1) = y_smats(:,1)/4;
eps_z_vec = [0*eps_a(1:end) 0*eps_e(1:end) eps_z(1:end) 0*eps_r(1:end)]';  
for t = 2:T;
    y_eps_z(:,t) = S1*y_eps_z(:,t-1)+S2*eps_z_vec(:,t);
end
% Only policy shocks
y_eps_r = zeros(nvars,T);
y_eps_r(:,1) = y_smats(:,1)/4;
eps_r_vec = [0*eps_a(1:end) 0*eps_e(1:end) 0*eps_z(1:end) eps_r(1:end)]';  
for t = 2:T;
    y_eps_r(:,t) = S1*y_eps_r(:,t-1)+S2*eps_r_vec(:,t);
end
figure(1)
subplot(3,1,1),plot(time,y_eps_a(5,:),'b',time,ppihat,'r'), legend('\epsilon_a only','data'), title('Inflation')
subplot(3,1,2),plot(time,y_eps_a(6,:),'b',time,ghat,'r'), legend('\epsilon_a only','data'), title('Growth')
subplot(3,1,3),plot(time,y_eps_a(8,:),'b',time,rhat,'r'), legend('\epsilon_a only','data'), title('Interest Rate')
figure(2)
subplot(3,1,1),plot(time,y_eps_e(5,:),'b',time,ppihat,'r'), legend('\epsilon_e only','data'), title('Inflation')
subplot(3,1,2),plot(time,y_eps_e(6,:),'b',time,ghat,'r'), legend('\epsilon_e only','data'), title('Growth')
subplot(3,1,3),plot(time,y_eps_e(8,:),'b',time,rhat,'r'), legend('\epsilon_e only','data'), title('Interest Rate')
figure(3)
subplot(3,1,1),plot(time,y_eps_z(5,:),'b',time,ppihat,'r'), legend('\epsilon_z only','data'), title('Inflation')
subplot(3,1,2),plot(time,y_eps_z(6,:),'b',time,ghat,'r'), legend('\epsilon_z only','data'), title('Growth')
subplot(3,1,3),plot(time,y_eps_z(8,:),'b',time,rhat,'r'), legend('\epsilon_z only','data'), title('Interest Rate')
figure(4)
subplot(3,1,1),plot(time,y_eps_r(5,:),'b',time,ppihat,'r'), legend('\epsilon_r only','data'), title('Inflation')
subplot(3,1,2),plot(time,y_eps_r(6,:),'b',time,ghat,'r'), legend('\epsilon_r only','data'), title('Growth')
subplot(3,1,3),plot(time,y_eps_r(8,:),'b',time,rhat,'r'), legend('\epsilon_r only','data'), title('Interest Rate')
end

%% Check decompositions add to data
if checksum
ysum = y_eps_a + y_eps_e + y_eps_r + y_eps_z;
figure(5)
plot(time,ysum(5,:),'b',time,ppihat,'r'), title('Inflation')
figure(6)
plot(time,ysum(6,:),'b',time,ghat,'r'), title('Growth')
figure(7)
plot(time,ysum(8,:),'b',time,rhat,'r'), title('Interest Rate')
end







